/* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
 *
 * Copyright 1998-2005 Decklin Foster <decklin@red-bean.com>. This
 * program is free software; please see LICENSE for details. */

#include <X11/StringDefs.h>
#include <X11/Intrinsic.h>
#include <X11/Xaw/SimpleMenu.h>
#include <X11/Xaw/SmeBSB.h>
#include "aeclients.h"
#include "atom.h"
#include "launch.h"
#include "switch.h"
#include "xaw-util.h"

void make_launch_menu_item(menu_t, char *, char *); 
menu_t make_launch_sub_menu(menu_t, char *);
void make_client_menu_item(long *, void *);
void raise_win_cb(Widget, XtPointer, XtPointer);
void fork_exec_cb(Widget, XtPointer, XtPointer);

int main(int argc, char **argv)
{
    Widget toplevel, main_menu;
    int i, mode = LAUNCH;
    char *opt_config = NULL;

    setlocale(LC_ALL, "");
    toplevel = XtInitialize(argv[0], "AeMenu", NULL, 0, &argc, argv);

    for (i = 1; i < argc; i++) {
        if ARG("config", "rc", 1)  {
            opt_config = argv[++i];
        } else if ARG("launch", "l", 0)   {
            mode = LAUNCH;
        } else if ARG("switch", "s", 0)   {
            mode = SWITCH;
        } else {
            fprintf(stderr, "usage: aemenu [--switch|-s] [--config|-rc <file>]\n");
            exit(2);
        }
    }

    dpy = XtDisplay(toplevel);
    root = DefaultRootWindow(dpy);
    main_menu = XtVaCreatePopupShell("menu",
        simpleMenuWidgetClass, toplevel, NULL);

    if (mode == LAUNCH) {
        make_launch_menu(opt_config, main_menu,
            make_launch_menu_item, make_launch_sub_menu);
    } else /* mode == SWITCH */ {
        setup_switch_atoms();
        atom_foreach(root, net_client_list, XA_WINDOW,
            make_client_menu_item, main_menu);
    }

    popup_cb(NULL, main_menu, NULL);
    XtMainLoop();
    return 0;
}

void make_launch_menu_item(menu_t menu, char *label, char *cmd)
{
    Widget menu_item = XtVaCreateManagedWidget(label,
        smeBSBObjectClass, menu, NULL);
    XtAddCallback(menu_item, XtNcallback, fork_exec_cb, cmd);
}

menu_t make_launch_sub_menu(menu_t menu, char *label)
{
    Widget new_menu, menu_item;
    new_menu = XtVaCreatePopupShell(label,
        simpleMenuWidgetClass, menu, NULL);
    menu_item = XtVaCreateManagedWidget(label,
        smeBSBObjectClass, menu,
        XtNmenuName, label, NULL);
    XtAddCallback(menu_item,
        XtNcallback, popup_cb, (XtPointer)new_menu);
    return new_menu;
}

void make_client_menu_item(long *prop_data, void *cb_data)
{
    Window w = *prop_data;
    Widget menu = cb_data;
    Widget menu_item;
    char buf[BUF_SIZE];

    if (is_on_cur_desk(w) && !is_skip(w)) {
        format_wm_name(w, buf, sizeof buf);
        menu_item = XtVaCreateManagedWidget(buf,
            smeBSBObjectClass, menu, NULL);
        XtAddCallback(menu_item,
            XtNcallback, raise_win_cb, (XtPointer)w);
    }
}

void raise_win_cb(Widget w, XtPointer data, XtPointer call)
{
    raise_win((Window)data);
    exit(0);
}

void fork_exec_cb(Widget w, XtPointer data, XtPointer call)
{
    fork_exec(data);
    exit(0);
}
