/* aewm - a minimalist X11 window mananager. vim:sw=4:ts=4:et
 *
 * Copyright 1998-2005 Decklin Foster <decklin@red-bean.com>. This
 * program is free software; please see LICENSE for details. */

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <sys/types.h>
#include "switch.h"
#include "atom.h"

Display *dpy;
Window root;
Atom wm_state;
Atom utf8_string;
Atom net_client_list;
Atom net_current_desktop;
Atom net_wm_name;
Atom net_wm_desktop;
Atom net_wm_state;
Atom net_wm_state_skip_taskbar;
Atom net_wm_state_skip_pager;

void setup_switch_atoms()
{
    wm_state = XInternAtom(dpy,
        "WM_STATE", False);
    utf8_string = XInternAtom(dpy,
        "UTF8_STRING", False);
    net_client_list = XInternAtom(dpy,
        "_NET_CLIENT_LIST", False);
    net_current_desktop = XInternAtom(dpy,
        "_NET_CURRENT_DESKTOP", False);
    net_wm_name = XInternAtom(dpy,
        "_NET_WM_NAME", False);
    net_wm_desktop = XInternAtom(dpy,
        "_NET_WM_DESKTOP", False);
    net_wm_state = XInternAtom(dpy,
        "_NET_WM_STATE", False);
    net_wm_state_skip_taskbar = XInternAtom(dpy,
        "_NET_WM_STATE_SKIP_TASKBAR", False);
    net_wm_state_skip_pager = XInternAtom(dpy,
        "_NET_WM_STATE_SKIP_PAGER", False);
}

long get_wm_state(Window w)
{
    long state;

    if (get_atom(w, wm_state, wm_state, &state))
        return state;
    else
        return WithdrawnState;
}

/* XXX: do we need this anymore? maybe just snprintf it */

void format_wm_name(Window w, char *buf, size_t len)
{
    char *name;

    name = get_wm_name(w);

    if (name) {
        if (get_wm_state(w) == NormalState) {
            if (strlen(name) >= len) strcpy(name+len-4, "...");
            strcpy(buf, name);
        } else {
            if (strlen(name) >= len-2) strcpy(name+len-6, "...");
            sprintf(buf, "[%s]", name);
        }
        XFree(name);
    } else {
        strncpy(buf, "(null)", len);
    }
}

/* This is only right if the WM has already received the ClientMessage
 * and processed it and set the property in response and nothing else
 * has fucked with it in the interim. Don't blame me, blame the spec.
 *
 * aepanel ignores the ClientMessage and lets the WM answer it. The WM
 * then actually sets the property, and our PropertyNotify handler will
 * call this. This means that if some other misbehaving client simply
 * changes the prop, we will get seriously confused. The alternative
 * would be to not use this code at all, select for
 * SubstructureNotifyMask, and handle the ClientMessage, but that would
 * require knowing the WM's desk policy and reimplementing it (since
 * there's no way to tell who will get called first), which is rather
 * intractable. Perhaps I'm just missing something in the spec.
 *
 * If we're walking into a party that's already started, though (as is
 * the case for aemenu), it's all we got. Wing and a prayer. */

int is_on_cur_desk(Window w)
{
    long cur_desk, w_desk;

    if (get_atom(root, net_current_desktop, XA_CARDINAL, &cur_desk) &&
            get_atom(w, net_wm_desktop, XA_CARDINAL, &w_desk))
        return IS_ON_DESK(w_desk, cur_desk);
    else
        return 1;
}

int is_skip(Window w)
{
    int skippy = 0;

    atom_foreach(w, net_wm_state, XA_ATOM, test_skip, &skippy);
    return skippy;
}

void test_skip(long *atom, void *cb_data)
{
    Atom prop = *atom;
    int *skip = cb_data;

    if (prop == net_wm_state_skip_taskbar || prop == net_wm_state_skip_pager)
        *skip = 1;
}

void raise_win(Window w)
{
    XMapRaised(dpy, w);
    XSync(dpy, False);
}
